package sun.text.normalizer;

import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStream;

public class CharTrie extends Trie
{
    private char m_initialValue_;
    private char[] m_data_;
    private CharTrie.FriendAgent m_friendAgent_;
    
    public CharTrie(InputStream inputStream, Trie.DataManipulate dataManipulate) throws IOException {
        super(inputStream, dataManipulate);
        if (!this.isCharTrie()) {
            throw new IllegalArgumentException("Data given does not belong to a char trie.");
        }
        this.m_friendAgent_ = new CharTrie.FriendAgent();
        return;
    }
    
    public CharTrie(int n, int n2, Trie.DataManipulate dataManipulate) {
        super(new char[2080], 512, dataManipulate);
        int dataLength_;
        int i = dataLength_ = 256;
        if (n2 != n) {
            dataLength_ += 32;
        }
        this.m_data_ = new char[dataLength_];
        this.m_dataLength_ = dataLength_;
        this.m_initialValue_ = (char)n;
        for (int j = 0; j < i; ++j) {
            this.m_data_[j] = (char)n;
        }
        if (n2 != n) {
            int n3 = (char)(i >> 2);
            for (int k = 1728; k < 1760; ++k) {
                this.m_index_[k] = n3;
            }
            int n4;
            for (n4 = i + 32, ; i < n4; ++i) {
                this.m_data_[i] = (char)n2;
            }
        }
        this.m_friendAgent_ = new CharTrie.FriendAgent();
    }
    
    public void putIndexData(UCharacterProperty uCharacterProperty) {
        uCharacterProperty.setIndexData(this.m_friendAgent_);
    }
    
    public final char getCodePointValue(int n) {
        if (0 <= n && n < 55296) {
            return this.m_data_[(this.m_index_[n >> 5] << 2) + (n & 0x1F)];
        }
        int codePointOffset = this.getCodePointOffset(n);
        return (codePointOffset >= 0) ? this.m_data_[codePointOffset] : this.m_initialValue_;
    }
    
    public final char getLeadValue(char c) {
        return this.m_data_[this.getLeadOffset(c)];
    }
    
    public final char getSurrogateValue(char c, char c2) {
        int surrogateOffset = this.getSurrogateOffset(c, c2);
        if (surrogateOffset > 0) {
            return this.m_data_[surrogateOffset];
        }
        return this.m_initialValue_;
    }
    
    public final char getTrailValue(int n, char c) {
        if (this.m_dataManipulate_ == null) {
            throw new NullPointerException("The field DataManipulate in this Trie is null");
        }
        int foldingOffset = this.m_dataManipulate_.getFoldingOffset(n);
        if (foldingOffset > 0) {
            return this.m_data_[this.getRawOffset(foldingOffset, (char)(c & 'Ͽ'))];
        }
        return this.m_initialValue_;
    }
    
    @Override
    protected final void unserialize(InputStream in) throws IOException {
        DataInputStream dataInputStream = new DataInputStream(in);
        int n = this.m_dataOffset_ + this.m_dataLength_;
        this.m_index_ = new char[n];
        for (int i = 0; i < n; ++i) {
            this.m_index_[i] = dataInputStream.readChar();
        }
        this.m_data_ = this.m_index_;
        this.m_initialValue_ = this.m_data_[this.m_dataOffset_];
    }
    
    @Override
    protected final int getSurrogateOffset(char c, char c2) {
        if (this.m_dataManipulate_ == null) {
            throw new NullPointerException("The field DataManipulate in this Trie is null");
        }
        int foldingOffset = this.m_dataManipulate_.getFoldingOffset((int)this.getLeadValue(c));
        if (foldingOffset > 0) {
            return this.getRawOffset(foldingOffset, (char)(c2 & 'Ͽ'));
        }
        return -1;
    }
    
    @Override
    protected final int getValue(int n) {
        return this.m_data_[n];
    }
    
    @Override
    protected final int getInitialValue() {
        return this.m_initialValue_;
    }
    
    public class FriendAgent
    {
        public char[] getPrivateIndex() {
            return CharTrie.this.m_index_;
        }
        
        public char[] getPrivateData() {
            return CharTrie.this.m_data_;
        }
        
        public int getPrivateInitialValue() {
            return CharTrie.this.m_initialValue_;
        }
    }
}
